@@ -17,12 +17,15 @@ import copy
 import functools
 import json
 import re
+import uuid
+
+from castellan.common import exception as castellan_exception
 
 from django.conf import settings
 from django.contrib import auth
 from django.contrib.auth import decorators as auth_dec
 from django.contrib.staticfiles.templatetags.staticfiles import static
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 # django.contrib.formtools migration to django 1.8
 # https://docs.djangoproject.com/en/1.8/ref/contrib/formtools/
 try:
@@ -41,6 +44,9 @@ from horizon.forms import views
 from horizon import messages
 from horizon import tabs
 from horizon import views as generic_views
+from novaclient import exceptions as nova_exceptions
+from openstack_dashboard.api import nova
+from openstack_dashboard.usage import quotas
 from oslo_log import log as logging
 import six
 
@@ -120,7 +126,7 @@ def get_categories_list(request):
 @auth_dec.login_required
 def switch(request, environment_id,
            redirect_field_name=auth.REDIRECT_FIELD_NAME):
-    redirect_to = request.REQUEST.get(redirect_field_name, '')
+    redirect_to = request.GET.get(redirect_field_name, '')
     if not http_utils.is_safe_url(url=redirect_to, host=request.get_host()):
         redirect_to = settings.LOGIN_REDIRECT_URL
 
@@ -245,7 +251,12 @@ def quick_deploy(request, app_id):
 
 
 def get_image(request, app_id):
-    content = pkg_api.get_app_logo(request, app_id)
+    try:
+        content = pkg_api.get_app_logo(request, app_id)
+    except (AttributeError, exc.HTTPNotFound):
+        message = _("Can not get logo for {0}.").format(app_id)
+        LOG.warning(message)
+        content = None
     if content:
         return http.HttpResponse(content=content, content_type='image/png')
     else:
@@ -254,7 +265,12 @@ def get_image(request, app_id):
 
 
 def get_supplier_image(request, app_id):
-    content = pkg_api.get_app_supplier_logo(request, app_id)
+    try:
+        content = pkg_api.get_app_supplier_logo(request, app_id)
+    except (AttributeError, exc.HTTPNotFound):
+        message = _("Can not get supplier logo for {0}.").format(app_id)
+        LOG.warning(message)
+        content = None
     if content:
         return http.HttpResponse(content=content, content_type='image/png')
     else:
@@ -331,14 +347,22 @@ class Wizard(generic_views.PageTitleMixin, views.ModalFormMixin, LazyWizard):
 
     def done(self, form_list, **kwargs):
         app_name = self.storage.extra_data['app'].name
-
-        service = form_list[0].service
-        attributes = service.extract_attributes()
+        service = tuple(form_list)[0].service
+        try:
+            attributes = service.extract_attributes()
+        except castellan_exception.KeyManagerError as e:
+            msg = _("This Application requires encryption, please contact "
+                    "your administrator to configure this.")
+            messages.error(self.request, msg)
+            LOG.exception(e)
+            raise
         attributes = helpers.insert_hidden_ids(attributes)
 
         storage = attributes.setdefault('?', {}).setdefault(
             consts.DASHBOARD_ATTRS_KEY, {})
         storage['name'] = app_name
+        attributes['?']['resourceUsages'] = self.aggregate_usages(
+            self.init_usages()[1:])
 
         do_redirect = self.get_wizard_flag('do_redirect')
         wm_form_data = service.cleaned_data.get('workflowManagement')
@@ -346,7 +370,7 @@ class Wizard(generic_views.PageTitleMixin, views.ModalFormMixin, LazyWizard):
             do_redirect = do_redirect or not wm_form_data.get(
                 'stay_at_the_catalog', True)
 
-        fail_url = reverse("horizon:murano:environments:index")
+        fail_url = reverse("horizon:app-catalog:environments:index")
         environment_id = utils.ensure_python_obj(kwargs.get('environment_id'))
         quick_environment_id = self.request.session.get('quick_env_id')
         try:
@@ -360,7 +384,7 @@ class Wizard(generic_views.PageTitleMixin, views.ModalFormMixin, LazyWizard):
                     environment_id = env.id
                 else:
                     environment_id = quick_environment_id
-            env_url = reverse('horizon:murano:environments:services',
+            env_url = reverse('horizon:app-catalog:environments:services',
                               args=(environment_id,))
 
             srv = env_api.service_create(
@@ -374,7 +398,7 @@ class Wizard(generic_views.PageTitleMixin, views.ModalFormMixin, LazyWizard):
             LOG.exception(message)
             if quick_environment_id:
                 env_api.environment_delete(self.request, quick_environment_id)
-                fail_url = reverse('horizon:murano:catalog:index')
+                fail_url = reverse('horizon:app-catalog:catalog:index')
             exceptions.handle(self.request, message, redirect=fail_url)
         else:
             message = _("The '{0}' application successfully added to "
@@ -383,7 +407,7 @@ class Wizard(generic_views.PageTitleMixin, views.ModalFormMixin, LazyWizard):
             messages.success(self.request, message)
 
             if do_redirect:
-                return http.HttpResponseRedirect(env_url)
+                return http.HttpResponseRedirect(bytes(env_url))
             else:
                 srv_id = getattr(srv, '?')['id']
                 return self.create_hacked_response(
@@ -421,6 +445,122 @@ class Wizard(generic_views.PageTitleMixin, views.ModalFormMixin, LazyWizard):
         value = self._get_wizard_param(key)
         return utils.ensure_python_obj(value)
 
+    def get_flavors(self):
+        try:
+            flavors = nova.flavor_list(self.request)
+        except nova_exceptions.ClientException:
+            message = _("Failed to get list of flavors.")
+            exceptions.handle(self.request, message)
+            LOG.exception(message)
+            flavors = []
+
+        def extract(flavor):
+            info = flavor._info
+            return {k: v for (k, v) in info.items() if k != 'links'}
+        flavors = [extract(f) for f in flavors]
+        self.storage.extra_data['flavors'] = flavors
+        return json.dumps(flavors)
+
+    def get_flavor_usages(self, form):
+        selected_flavor = form.cleaned_data['flavor']
+        for flavor in self.storage.extra_data['flavors']:
+            if flavor['name'] == selected_flavor:
+                return {'ram': flavor['ram'],
+                        'vcpus': flavor['vcpus'],
+                        'instances': 1}
+
+    def init_usages(self):
+        stored_data = self.storage.extra_data
+        step_usages = stored_data.get('step_usages')
+        if step_usages is None:
+            step_usages = [
+                collections.defaultdict(dict)
+                for step in self.steps.all
+            ]
+            stored_data['step_usages'] = step_usages
+
+            environment_id = self.kwargs.get('environment_id')
+            environment_id = utils.ensure_python_obj(environment_id)
+            if environment_id is not None:
+                session_id = env_api.Session.get(self.request, environment_id)
+                client = api.muranoclient(self.request)
+                all_services = client.environments.get(
+                    environment_id, session_id).services
+                env_usages = self.aggregate_usages(map(
+                    lambda svc: svc['?'].get('resourceUsages', {}),
+                    all_services))
+            else:
+                env_usages = collections.defaultdict(dict)
+            step_usages.insert(0, env_usages)
+
+        return step_usages
+
+    def process_step(self, form):
+        data = super(Wizard, self).process_step(form)
+        region = form.region or self.request.user.services_region
+        step_usages = self.init_usages()
+        if 'flavor' in form.cleaned_data:
+            usages = self.get_flavor_usages(form)
+            step_usages[self.steps.step0 + 1][region].update({
+                'ram': usages['ram'],
+                'vcpus': usages['vcpus'],
+                'instances': usages['instances']
+            })
+        else:
+            step_usages[self.steps.step0 + 1][region].update({
+                'ram': 0,
+                'vcpus': 0,
+                'instances': 0
+            })
+
+        return data
+
+    def update_usages(self, form, context):
+        data = self.init_usages()
+        usages = quotas.tenant_quota_usages(self.request).usages
+        region = self.request.user.services_region
+        inf = float('inf')
+
+        def get_usage(group, name, default):
+            return usages.get(group, {}).get(name, default)
+
+        context.update({
+            'usages': {
+                'maxTotalInstances': get_usage('instances', 'quota', inf),
+                'totalInstancesUsed': get_usage('instances', 'used', 0),
+                'maxTotalCores': get_usage('cores', 'quota', inf),
+                'totalCoresUsed': get_usage('cores', 'used', 0),
+                'maxTotalRAMSize': get_usage('ram', 'quota', inf),
+                'totalRAMUsed': get_usage('ram', 'used', 0),
+            },
+            'other_usages': {},
+            'flavors': self.get_flavors(),
+            'contexts': ['', 'info', 'success']
+        })
+        for step in range(self.steps.step0 + 1):
+
+            def sum_usage(context_key, data_key):
+                if context_key not in context['other_usages']:
+                    context['other_usages'][context_key] = 0
+                context['other_usages'][context_key] += \
+                    data[step][region].get(data_key, 0)
+            sum_usage('totalInstancesUsed', 'instances')
+            sum_usage('totalCoresUsed', 'vcpus')
+            sum_usage('totalRAMUsed', 'ram')
+
+        return context
+
+    @staticmethod
+    def aggregate_usages(steps):
+        result = collections.defaultdict(dict)
+        for step in steps:
+            for region, region_usages in six.iteritems(step):
+                for metric, value in six.iteritems(region_usages):
+                    if metric not in result[region]:
+                        result[region][metric] = 0
+                    result[region][metric] += value
+        return result
+
     def get_context_data(self, form, **kwargs):
         context = super(Wizard, self).get_context_data(form=form, **kwargs)
         mc = api.muranoclient(self.request)
@@ -432,6 +572,10 @@ class Wizard(generic_views.PageTitleMixin, views.ModalFormMixin, LazyWizard):
             app = mc.packages.get(app_id)
             self.storage.extra_data['app'] = app
 
+        wizard_id = self.request.POST.get('wizard_id')
+        if wizard_id is None:
+            wizard_id = uuid.uuid4()
+
         environment_id = self.kwargs.get('environment_id')
         environment_id = utils.ensure_python_obj(environment_id)
         if environment_id is not None:
@@ -450,9 +594,12 @@ class Wizard(generic_views.PageTitleMixin, views.ModalFormMixin, LazyWizard):
                         'do_redirect': self.get_wizard_flag('do_redirect'),
                         'drop_wm_form': self.get_wizard_flag('drop_wm_form'),
                         'prefix': self.prefix,
+                        'wizard_id': wizard_id,
                         'field_descriptions': field_descr,
                         'extended_descriptions': extended_descr,
                         })
+        with helpers.current_region(self.request, form.region):
+            context = self.update_usages(form, context)
         return context
 
 
@@ -554,21 +701,21 @@ class IndexView(generic_views.PageTitleMixin, list_view.ListView):
             query_params['marker'] = marker
         if sort_dir:
             query_params['sort_dir'] = sort_dir
-        return '{0}?{1}'.format(reverse('horizon:murano:catalog:index'),
+        return '{0}?{1}'.format(reverse('horizon:app-catalog:catalog:index'),
                                 http_utils.urlencode(query_params))
 
     def prev_page_url(self):
         query_params = self.get_query_params()
         query_params['marker'] = self.get_marker(0)
         query_params['sort_dir'] = 'desc'
-        return '{0}?{1}'.format(reverse('horizon:murano:catalog:index'),
+        return '{0}?{1}'.format(reverse('horizon:app-catalog:catalog:index'),
                                 http_utils.urlencode(query_params))
 
     def next_page_url(self):
         query_params = self.get_query_params()
         query_params['marker'] = self.get_marker()
         query_params['sort_dir'] = 'asc'
-        return '{0}?{1}'.format(reverse('horizon:murano:catalog:index'),
+        return '{0}?{1}'.format(reverse('horizon:app-catalog:catalog:index'),
                                 http_utils.urlencode(query_params))
 
     def get_context_data(self, **kwargs):
@@ -588,7 +735,7 @@ class IndexView(generic_views.PageTitleMixin, list_view.ListView):
         context['tenant_id'] = self.request.session['token'].tenant['id']
         context.update(get_environments_context(self.request))
         context['display_repo_url'] = pkg_consts.DISPLAY_MURANO_REPO_URL
-        context['pkg_def_url'] = reverse('horizon:murano:packages:index')
+        context['pkg_def_url'] = reverse('horizon:app-catalog:packages:index')
         context['no_apps'] = True
         if self.get_current_category() != ALL_CATEGORY_NAME or search:
             context['no_apps'] = False
