@@ -17,29 +17,31 @@ import copy
 import json
 import re
 
-from django.core.urlresolvers import reverse
 from django.core import validators as django_validator
 from django import forms
 from django.forms import widgets
 from django.template import defaultfilters
+from django.urls import reverse
 from django.utils.encoding import force_text
 from django.utils import html
 from django.utils.translation import ugettext_lazy as _
 from horizon import exceptions
 from horizon import forms as hz_forms
 from horizon import messages
+from openstack_dashboard.api import cinder
 from openstack_dashboard.api import glance
+from openstack_dashboard.api import neutron
 from openstack_dashboard.api import nova
 from oslo_log import log as logging
+from oslo_log import versionutils
 import six
 from yaql import legacy
 
 from muranodashboard.api import packages as pkg_api
 from muranodashboard.common import net
+from muranodashboard.dynamic_ui import helpers
 from muranodashboard.environments import api as env_api
 
-from oslo_log import versionutils
-
 
 LOG = logging.getLogger(__name__)
 
@@ -105,19 +107,25 @@ def wrap_regex_validator(validator, message):
     return _validator
 
 
-def get_murano_images(request):
+def get_murano_images(request, region=None):
     images = []
     try:
         # https://bugs.launchpad.net/murano/+bug/1339261 - glance
         # client version change alters the API. Other tuple values
         # are _more and _prev (in recent glance client)
-        images = glance.image_list_detailed(request)[0]
+        with helpers.current_region(request, region):
+            images = glance.image_list_detailed(request)[0]
     except Exception:
         LOG.error("Error to request image list from glance ")
         exceptions.handle(request, _("Unable to retrieve public images."))
     murano_images = []
+    # filter out the snapshot image type
+    images = filter(
+        lambda x: x.properties.get("image_type", '') != 'snapshot', images)
     for image in images:
-        murano_property = image.properties.get('murano_image_info')
+        # Additional properties, whose value is always a string data type, are
+        # only included in the response if they have a value.
+        murano_property = getattr(image, 'murano_image_info', None)
         if murano_property:
             try:
                 murano_metadata = json.loads(murano_property)
@@ -135,17 +143,23 @@ class RawProperty(object):
     def __init__(self, key, spec):
         self.key = key
         self.spec = spec
+        self.value = None
+        self.value_evaluated = False
 
-    def finalize(self, form_name, service):
+    def finalize(self, form_name, service, cls):
         def _get(field):
-            data_ready, value = service.get_data(form_name, self.spec)
-            return value if data_ready else field.__dict__[self.key]
+            if self.value_evaluated:
+                return self.value
+            return service.get_data(form_name, self.spec)
 
         def _set(field, value):
-            field.__dict__[self.key] = value
+            self.value = value
+            self.value_evaluated = value is not None
+            if hasattr(cls, self.key):
+                getattr(cls, self.key).fset(field, value)
 
         def _del(field):
-            del field.__dict__[self.key]
+            _set(field, None)
         return property(_get, _set, _del)
 
 
@@ -153,6 +167,8 @@ FIELD_ARGS_TO_ESCAPE = ['help_text', 'initial', 'description', 'label']
 
 
 class CustomPropertiesField(forms.Field):
+    js_validation = False
+
     def __init__(self, description=None, description_title=None,
                  *args, **kwargs):
         self.description = description
@@ -164,21 +180,46 @@ class CustomPropertiesField(forms.Field):
                 kwargs[arg] = html.escape(force_text(kwargs[arg]))
 
         validators = []
+        validators_js = []
         for validator in kwargs.get('validators', []):
             if hasattr(validator, '__call__'):  # single regexpValidator
                 validators.append(validator)
-            else:  # mixed list of regexpValidator-s and YAQL validators
+                if hasattr(validator, 'regex'):
+                    regex_message = ''
+                    error_messages = kwargs.get('error_messages', {})
+                    if hasattr(validator, 'code') and \
+                       validator.code in error_messages:
+                        regex_message = force_text(
+                            error_messages[validator.code]
+                        )
+                    validators_js. \
+                        append({'regex': force_text(validator.regex.pattern),
+                                'message': regex_message})
+            else:  # mixed list of regexpValidator and YAQL validators
                 expr = validator.get('expr')
                 regex_validator = get_regex_validator(expr)
+                regex_message = validator.get('message', '')
                 if regex_validator:
                     validators.append(wrap_regex_validator(
-                        regex_validator, validator.get('message', '')))
+                        regex_validator, regex_message))
                 elif isinstance(expr, RawProperty):
                     validators.append(validator)
+                if hasattr(regex_validator, 'regex'):
+                    validators_js.\
+                        append({'regex': regex_validator.regex.pattern,
+                                'message': regex_message})
         kwargs['validators'] = validators
+        if validators_js:
+            self.js_validation = json.dumps(validators_js)
 
         super(CustomPropertiesField, self).__init__(*args, **kwargs)
 
+    def widget_attrs(self, widget):
+        attrs = super(CustomPropertiesField, self).widget_attrs(widget)
+        if self.js_validation:
+            attrs['data-validators'] = self.js_validation
+        return attrs
+
     def clean(self, value):
         """Skip all validators if field is disabled."""
         # form is assigned in ServiceConfigurationForm.finalize_fields()
@@ -194,9 +235,10 @@ class CustomPropertiesField(forms.Field):
     @classmethod
     def finalize_properties(cls, kwargs, form_name, service):
         props = {}
-        for key, value in kwargs.items():
+        kwargs_ = copy.copy(kwargs)
+        for key, value in kwargs_.items():
             if isinstance(value, RawProperty):
-                props[key] = value.finalize(form_name, service)
+                props[key] = value.finalize(form_name, service, cls)
                 del kwargs[key]
         if props:
             return type(cls.__name__, (cls,), props)
@@ -214,6 +256,7 @@ class PasswordField(CharField):
                              % special_characters)
     has_clone = False
     original = True
+    attrs = {'data-type': 'password'}
     validate_password = django_validator.RegexValidator(
         password_re, _('The password must contain at least one letter, one   \
                                number and one special character'), 'invalid')
@@ -230,16 +273,13 @@ class PasswordField(CharField):
                 raise forms.ValidationError(_(u"{0}{1} don't match").format(
                     self.label, defaultfilters.pluralize(2)))
 
-    class PasswordInput(forms.PasswordInput):
-        class Media(object):
-            js = ('muranodashboard/js/passwordfield.js',)
-
     def __init__(self, label, *args, **kwargs):
         self.confirm_input = kwargs.pop('confirm_input', True)
 
         kwargs.update({'label': label,
                        'error_messages': kwargs.get('error_messages', {}),
-                       'widget': forms.PasswordInput(render_value=True)})
+                       'widget': forms.PasswordInput(attrs=self.attrs,
+                                                     render_value=True)})
 
         validators = kwargs.get('validators')
         help_text = kwargs.get('help_text')
@@ -256,7 +296,8 @@ class PasswordField(CharField):
                 'invalid', self.validate_password.message)
             kwargs['min_length'] = kwargs.get('min_length', 7)
             kwargs['max_length'] = kwargs.get('max_length', 255)
-            kwargs['widget'] = self.PasswordInput(render_value=True)
+            kwargs['widget'] = forms.PasswordInput(attrs=self.attrs,
+                                                   render_value=True)
         else:
             if not help_text:
                 # NOTE(kzaitsev) There are custom validators for password,
@@ -289,28 +330,58 @@ class IntegerField(forms.IntegerField, CustomPropertiesField):
     pass
 
 
+def _get_title(data):
+    if isinstance(data, Choice):
+        return data.title
+    return data
+
+
+def _disable_non_ready(data):
+    if getattr(data, 'enabled', True):
+        return {}
+    else:
+        return {'disabled': 'disabled'}
+
+
 class ChoiceField(forms.ChoiceField, CustomPropertiesField):
-    pass
+    def __init__(self, **kwargs):
+        choices = kwargs.get('choices') or getattr(self, 'choices', None)
+        if choices:
+            if isinstance(choices, dict):
+                choices = list(choices.items())
+            kwargs['choices'] = choices
+        super(ChoiceField, self).__init__(**kwargs)
 
 
 class DynamicChoiceField(hz_forms.DynamicChoiceField, CustomPropertiesField):
     pass
 
 
+class FlavorWidget(widgets.Select):
+    def __init__(self, *args, **kwargs):
+        super(FlavorWidget, self).__init__(*args, **kwargs)
+        self.attrs['class'] = self.attrs.get('class', '') + ' flavor'
+        self.attrs['id'] = 'id_flavor'
+
+
 class FlavorChoiceField(ChoiceField):
+    widget = FlavorWidget
+
     def __init__(self, *args, **kwargs):
         if 'requirements' in kwargs:
             self.requirements = kwargs.pop('requirements')
         super(FlavorChoiceField, self).__init__(*args, **kwargs)
 
     @with_request
-    def update(self, request, **kwargs):
-        self.choices = []
-        flavors = nova.novaclient(request).flavors.list()
+    def update(self, request, form=None, **kwargs):
+        choices = []
+        with helpers.current_region(request,
+                                    getattr(form, 'region', None)):
+            flavors = nova.novaclient(request).flavors.list()
 
         # If no requirements are present, return all the flavors.
         if not hasattr(self, 'requirements'):
-            self.choices = [(flavor.name, flavor.name) for flavor in flavors]
+            choices = [(flavor.id, flavor.name) for flavor in flavors]
         else:
             for flavor in flavors:
                 # If a flavor doesn't meet a minimum requirement,
@@ -331,21 +402,54 @@ class FlavorChoiceField(ChoiceField):
                 if 'max_memory_mb' in self.requirements:
                     if flavor.ram > self.requirements['max_memory_mb']:
                         continue
-                self.choices.append((flavor.name, flavor.name))
-        # Search through selected flavors
-        for flavor_name, flavor_name in self.choices:
-            if 'medium' in flavor_name:
-                self.initial = flavor_name
-                break
+                choices.append((flavor.id, flavor.name))
+
+        choices.sort(key=lambda e: e[1])
+        self.choices = choices
+        if kwargs.get('form'):
+            kwargs_form_flavor = kwargs["form"].fields.get('flavor')
+        else:
+            kwargs_form_flavor = None
+        if kwargs_form_flavor:
+            self.initial = kwargs["form"]["flavor"].value()
+        else:
+            # Search through selected flavors
+            for flavor_id, flavor_name in self.choices:
+                if 'medium' in flavor_name:
+                    self.initial = flavor_id
+                    break
+
+    def clean(self, value):
+        for flavor_id, flavor_name in self.choices:
+            if flavor_id == value:
+                return flavor_name
+        return value
 
 
 class KeyPairChoiceField(DynamicChoiceField):
-    " This widget allows to select keypair for VMs "
+    """This widget allows to select keypair for VMs"""
+    @with_request
+    def update(self, request, form=None, **kwargs):
+        self.choices = [('', _('No keypair'))]
+        with helpers.current_region(request,
+                                    getattr(form, 'region', None)):
+            keypairs = nova.novaclient(request).keypairs.list()
+        for keypair in sorted(keypairs, key=lambda e: e.name):
+            self.choices.append((keypair.name, keypair.name))
+
+
+class SecurityGroupChoiceField(DynamicChoiceField):
+    """This widget allows to select a security group for VMs"""
     @with_request
     def update(self, request, **kwargs):
-        self.choices = [('', _('No keypair'))]
-        for keypair in nova.novaclient(request).keypairs.list():
-            self.choices.append((keypair.name, keypair.name))
+        self.choices = [('', _('Application default security group'))]
+        # TODO(pbourke): remove sorted when supported natively in Horizon
+        # (https://bugs.launchpad.net/horizon/+bug/1692972)
+        for secgroup in sorted(
+                neutron.security_group_list(request),
+                key=lambda e: e.name_or_id):
+            if not secgroup.name_or_id.startswith('murano--'):
+                self.choices.append((secgroup.name_or_id, secgroup.name_or_id))
 
 
 # NOTE(kzaitsev): for transform to work correctly on horizon SelectWidget
@@ -357,19 +461,6 @@ class Choice(object):
         self.enabled = enabled
 
 
-def _get_title(data):
-    if isinstance(data, Choice):
-        return data.title
-    return data
-
-
-def _disable_non_ready(data):
-    if getattr(data, 'enabled', True):
-        return {}
-    else:
-        return {'disabled': 'disabled'}
-
-
 class ImageChoiceField(ChoiceField):
     widget = hz_forms.SelectWidget(transform=_get_title,
                                    transform_html_attrs=_disable_non_ready)
@@ -379,9 +470,10 @@ class ImageChoiceField(ChoiceField):
         super(ImageChoiceField, self).__init__(*args, **kwargs)
 
     @with_request
-    def update(self, request, **kwargs):
+    def update(self, request, form=None, **kwargs):
         image_map, image_choices = {}, []
-        murano_images = get_murano_images(request)
+        murano_images = get_murano_images(
+            request, getattr(form, 'region', None))
         for image in murano_images:
             murano_data = image.murano_property
             title = murano_data.get('title', image.name)
@@ -403,7 +495,8 @@ class ImageChoiceField(ChoiceField):
                     continue
             image_map[image.id] = title
 
-        for id_, title in sorted(six.iteritems(image_map), key=lambda e: e[1]):
+        for id_, title in sorted(six.iteritems(image_map),
+                                 key=lambda e: e[1].title):
             image_choices.append((id_, title))
         if image_choices:
             image_choices.insert(0, ("", _("Select Image")))
@@ -415,7 +508,6 @@ class ImageChoiceField(ChoiceField):
 
 class NetworkChoiceField(ChoiceField):
     def __init__(self,
-                 include_subnets=True,
                  filter=None,
                  murano_networks=None,
                  allow_auto=True,
@@ -426,7 +518,6 @@ class NetworkChoiceField(ChoiceField):
             if murano_networks.lower() not in ["exclude", "translate"]:
                 raise ValueError(_("Invalid value of 'murano_nets' option"))
         self.murano_networks = murano_networks
-        self.include_subnets = include_subnets
         self.allow_auto = allow_auto
         super(NetworkChoiceField, self).__init__(*args,
                                                  **kwargs)
@@ -439,7 +530,6 @@ class NetworkChoiceField(ChoiceField):
         rendered
         """
         network_choices = net.get_available_networks(request,
-                                                     self.include_subnets,
                                                      self.filter,
                                                      self.murano_networks)
         if self.allow_auto:
@@ -459,10 +549,12 @@ class NetworkChoiceField(ChoiceField):
 
 class AZoneChoiceField(ChoiceField):
     @with_request
-    def update(self, request, **kwargs):
+    def update(self, request, form=None, **kwargs):
         try:
-            availability_zones = nova.novaclient(
-                request).availability_zones.list(detailed=False)
+            with helpers.current_region(request,
+                                        getattr(form, 'region', None)):
+                availability_zones = nova.novaclient(
+                    request).availability_zones.list(detailed=False)
         except Exception:
             availability_zones = []
             exceptions.handle(request,
@@ -473,9 +565,52 @@ class AZoneChoiceField(ChoiceField):
         if not az_choices:
             az_choices.insert(0, ("", _("No availability zones available")))
 
+        az_choices.sort(key=lambda e: e[1])
         self.choices = az_choices
 
 
+class VolumeChoiceField(ChoiceField):
+    def __init__(self,
+                 include_volumes=True,
+                 include_snapshots=True,
+                 *args,
+                 **kwargs):
+        self.include_volumes = include_volumes
+        self.include_snapshots = include_snapshots
+        super(VolumeChoiceField, self).__init__(*args, **kwargs)
+
+    @with_request
+    def update(self, request, **kwargs):
+        """This widget allows selection of Volumes and Volume Snapshots"""
+        available = {'status': cinder.VOLUME_STATE_AVAILABLE}
+        choices = []
+
+        if self.include_volumes:
+            try:
+                choices.extend((volume.id, volume.name)
+                               for volume in cinder.volume_list(request,
+                               search_opts=available))
+            except Exception:
+                exceptions.handle(request,
+                                  _("Unable to retrieve volume list."))
+
+        if self.include_snapshots:
+            try:
+                choices.extend((snap.id, snap.name)
+                               for snap in cinder.volume_snapshot_list(request,
+                               search_opts=available))
+            except Exception:
+                exceptions.handle(request,
+                                  _("Unable to retrieve snapshot list."))
+
+        if choices:
+            choices.sort(key=lambda e: e[1])
+            choices.insert(0, ("", _("Select volume")))
+        else:
+            choices.insert(0, ("", _("No volumes available")))
+        self.choices = choices
+
+
 class BooleanField(forms.BooleanField, CustomPropertiesField):
     def __init__(self, *args, **kwargs):
         if 'widget' in kwargs:
@@ -595,7 +730,7 @@ def make_select_cls(fqns):
             # NOTE(kzaitsev): this closure is needed to allow us have custom
             # logic when clicking add button
             def _make_link():
-                ns_url = 'horizon:murano:catalog:add'
+                ns_url = 'horizon:app-catalog:catalog:add'
                 ns_url_args = (environment_id, False, True)
 
                 # This will prevent horizon from adding an extra '+' button
