@@ -15,8 +15,8 @@
 import json
 
 from django.contrib.staticfiles.templatetags.staticfiles import static
-from django.core.urlresolvers import reverse
 from django.template import loader
+from django.urls import reverse
 
 import six
 
@@ -25,7 +25,16 @@ from muranodashboard.environments import consts
 
 
 def get_app_image(request, app_fqdn, status=None):
-    package = pkg_cli.app_by_fqn(request, app_fqdn)
+    if '@' in app_fqdn:
+        class_fqn, package_fqn = app_fqdn.split('@')
+        if '/' in class_fqn:
+            class_fqn, version = class_fqn.split('/')
+        else:
+            version = None
+    else:
+        package_fqn = app_fqdn
+        version = None
+    package = pkg_cli.app_by_fqn(request, package_fqn, version=version)
     if status in [
        consts.STATUS_ID_DEPLOY_FAILURE,
        consts.STATUS_ID_DELETE_FAILURE,
@@ -37,7 +46,7 @@ def get_app_image(request, app_fqdn, status=None):
         url = static('dashboard/img/stack-gray.svg')
     if package:
         app_id = package.id
-        url = reverse("horizon:murano:catalog:images", args=(app_id,))
+        url = reverse("horizon:app-catalog:catalog:images", args=(app_id,))
     return url
 
 
@@ -141,6 +150,14 @@ def _create_ext_network_node(name):
     return node
 
 
+def _convert_lists(node_data):
+    for key, value in six.iteritems(node_data):
+        if isinstance(value, list) and all(
+                map(lambda s: not isinstance(s, (dict, list)), value)):
+            new_value = ', '.join(str(v) for v in value)
+            node_data[key] = new_value
+
+
 def _split_seq_by_predicate(seq, predicate):
     holds, not_holds = [], []
     for elt in seq:
@@ -193,6 +210,7 @@ def render_d3_data(request, environment):
     def rec(node_data, node_key, parent_node=None):
         if not isinstance(node_data, dict):
             return
+        _convert_lists(node_data)
         node_type = node_data.get('?', {}).get('type')
         node_id = node_data.get('?', {}).get('id')
         atomics, containers = _split_seq_by_predicate(
