@@ -14,8 +14,8 @@
 
 import itertools
 
-from django.core.urlresolvers import reverse
-from django.core.urlresolvers import reverse_lazy
+from django.urls import reverse
+from django.urls import reverse_lazy
 from django.utils.translation import ugettext_lazy as _
 
 from horizon import exceptions
@@ -62,24 +62,14 @@ class MarkedImagesView(horizon_tables.DataTableView):
         self._prev = False
         self._more = False
 
-        # TODO(kzaitsev) add v2 client support for marking images
-        try:
-            glance_v1_client = glance.glanceclient(self.request, "1")
-        except Exception:
-            # Horizon seems to raise ImportError which doesn't look
-            # specific enough. Let's catch any exceptions.
-            msg = _('Unable to create v1 glance client. Marking images '
-                    'from murano-dashboard will be unavailable.')
-            uri = reverse('horizon:murano:catalog:index')
-
-            exceptions.handle(self.request, msg, redirect=uri)
+        glance_v2_client = glance.glanceclient(self.request, "2")
 
         try:
-            images_iter = glance_v1_client.images.list(
+            images_iter = glance_v2_client.images.list(
                 **kwargs)
         except Exception:
             msg = _('Unable to retrieve list of images')
-            uri = reverse('horizon:murano:catalog:index')
+            uri = reverse('horizon:app-catalog:catalog:index')
 
             exceptions.handle(self.request, msg, redirect=uri)
 
@@ -112,6 +102,6 @@ class MarkImageView(views.ModalFormView):
     template_name = 'images/mark.html'
     context_object_name = 'image'
     page_title = _("Update Image")
-    success_url = reverse_lazy('horizon:murano:images:index')
+    success_url = reverse_lazy('horizon:app-catalog:images:index')
     submit_label = _('Mark Image')
-    submit_url = reverse_lazy('horizon:murano:images:mark_image')
+    submit_url = reverse_lazy('horizon:app-catalog:images:mark_image')
