@@ -16,9 +16,9 @@ import json
 import sys
 
 from django.core.files import storage
-from django.core.urlresolvers import reverse
-from django.core.urlresolvers import reverse_lazy
 from django import http
+from django.urls import reverse
+from django.urls import reverse_lazy
 from django.utils.translation import ugettext_lazy as _
 # django.contrib.formtools migration to django 1.8
 # https://docs.djangoproject.com/en/1.8/ref/contrib/formtools/
@@ -71,40 +71,16 @@ def is_app(wizard):
 
 
 def _ensure_images(name, package, request, step_data=None):
-    try:
-        glance_client = glance.glanceclient(
-            request, version='1')
-    except Exception:
-        glance_client = None
+    glance_client = glance.glanceclient(
+        request, version='2')
 
     base_url = packages_consts.MURANO_REPO_URL
     image_specs = package.images()
-    if not glance_client and len(image_specs):
-        # NOTE(kzaitsev): no glance_client. Probably v1 client
-        # is not available. Add warning, to let user know that
-        # we were unable to load images automagically
-        # since v2 does not have copy_from
-        download_urls = []
-        for image_spec in image_specs:
-            download_url = muranoclient_utils.to_url(
-                image_spec.get("Url", image_spec['Name']),
-                base_url=base_url,
-                path='images/',
-            )
-            download_urls.append(download_url)
-        msg = _("Couldn't initialise glance v1 client, "
-                "therefore could not download images for "
-                "'{0}' package. You may need to download them "
-                "manually from these locations: {1}").format(
-                    name, ' '.join(download_urls))
-        messages.error(request, msg)
-        LOG.error(msg)
-        return
 
     try:
         imgs = muranoclient_utils.ensure_images(
             glance_client=glance_client,
-            image_specs=package.images(),
+            image_specs=image_specs,
             base_url=base_url)
         for img in imgs:
             msg = _("Trying to add {0} image to glance. "
@@ -272,7 +248,7 @@ class ImportBundleWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
                 LOG.exception(msg)
                 messages.error(self.request, msg)
                 raise exceptions.Http302(
-                    reverse('horizon:murano:packages:index'))
+                    reverse('horizon:app-catalog:packages:index'))
 
             for package_spec in bundle.package_specs():
                 try:
@@ -329,11 +305,11 @@ class ImportBundleWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
         return step_data
 
     def done(self, form_list, **kwargs):
-        redirect = reverse('horizon:murano:packages:index')
+        redirect = reverse('horizon:app-catalog:packages:index')
         msg = _('Bundle successfully imported.')
         LOG.info(msg)
         messages.success(self.request, msg)
-        return http.HttpResponseRedirect(redirect)
+        return http.HttpResponseRedirect(bytes(redirect))
 
 
 class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
@@ -372,7 +348,7 @@ class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
         installed_images = self.storage.get_step_data('upload').get(
             'images', [])
 
-        redirect = reverse('horizon:murano:packages:index')
+        redirect = reverse('horizon:app-catalog:packages:index')
         dep_data = {'enabled': data['enabled'],
                     'is_public': data['is_public']}
         murano_client = api.muranoclient(self.request)
@@ -422,7 +398,7 @@ class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
             LOG.exception(msg)
             exceptions.handle(
                 self.request, msg,
-                redirect=reverse('horizon:murano:packages:index'))
+                redirect=reverse('horizon:app-catalog:packages:index'))
         except (exc.HTTPException, Exception):
             LOG.exception(_('Modifying package failed'))
             exceptions.handle(self.request,
@@ -432,7 +408,7 @@ class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
             msg = _('Package parameters successfully updated.')
             LOG.info(msg)
             messages.success(self.request, msg)
-            return http.HttpResponseRedirect(redirect)
+            return http.HttpResponseRedirect(bytes(redirect))
 
     def _handle_exception(self, original_e):
         exc_info = sys.exc_info()
@@ -444,13 +420,15 @@ class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
                     reason = error.get('message')
             except ValueError:
                 # Let horizon operate with original exception
-                raise exc_info[0], exc_info[1], exc_info[2]
+                six.reraise(original_e.__class__,
+                            original_e.__class__(original_e),
+                            exc_info[2])
         msg = _('Uploading package failed. {0}').format(reason)
         LOG.exception(msg)
         exceptions.handle(
             self.request,
             msg,
-            redirect=reverse('horizon:murano:packages:index'))
+            redirect=reverse('horizon:app-catalog:packages:index'))
 
     def process_step(self, form):
         @catalog_views.update_latest_apps
@@ -458,7 +436,7 @@ class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
             LOG.info('Adding {0} application to the'
                      ' latest apps list'.format(app_id))
 
-        step_data = self.get_form_step_data(form)
+        step_data = self.get_form_step_data(form).copy()
         if self.steps.current == 'upload':
             import_type = form.cleaned_data['import_type']
             data = {}
@@ -493,7 +471,7 @@ class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
                 LOG.exception(msg)
                 messages.error(self.request, msg)
                 raise exceptions.Http302(
-                    reverse('horizon:murano:packages:index'))
+                    reverse('horizon:app-catalog:packages:index'))
 
             reqs = package.requirements(base_url=base_url)
             original_package = reqs.pop(name)
@@ -545,7 +523,7 @@ class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
                 exceptions.handle(
                     self.request,
                     msg,
-                    redirect=reverse('horizon:murano:packages:index'))
+                    redirect=reverse('horizon:app-catalog:packages:index'))
             except exc.HTTPInternalServerError as e:
                 self._handle_exception(e)
 
@@ -558,7 +536,7 @@ class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
                 exceptions.handle(
                     self.request,
                     reason,
-                    redirect=reverse('horizon:murano:packages:index'))
+                    redirect=reverse('horizon:app-catalog:packages:index'))
 
             except Exception as original_e:
                 self._handle_exception(original_e)
@@ -571,15 +549,15 @@ class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
             kwargs.update({'request': self.request})
         if step == 'modify':
             package = self.storage.get_step_data('upload').get('package')
-            kwargs.update({'package': package})
+            kwargs.update({'package': package, 'request': self.request})
         return kwargs
 
 
 class ModifyPackageView(views.ModalFormView):
     form_class = forms.ModifyPackageForm
     template_name = 'packages/modify_package.html'
-    success_url = reverse_lazy('horizon:murano:packages:index')
-    failure_url = reverse_lazy('horizon:murano:packages:index')
+    success_url = reverse_lazy('horizon:app-catalog:packages:index')
+    failure_url = reverse_lazy('horizon:app-catalog:packages:index')
     page_title = _("Modify Package")
 
     def get_initial(self):
@@ -593,7 +571,7 @@ class ModifyPackageView(views.ModalFormView):
     def get_context_data(self, **kwargs):
         context = super(ModifyPackageView, self).get_context_data(**kwargs)
         context['app_id'] = self.kwargs['app_id']
-        context['type'] = kwargs['form'].initial['package'].type
+        context['type'] = self.get_form().initial['package'].type
         return context
 
 
@@ -608,11 +586,12 @@ class DetailView(horizon_views.HorizonTemplateView):
         return context
 
     def get_data(self):
+        app = None
         try:
             app_id = self.kwargs['app_id']
             app = api.muranoclient(self.request).packages.get(app_id)
         except Exception:
-            INDEX_URL = 'horizon:murano:packages:index'
+            INDEX_URL = 'horizon:app-catalog:packages:index'
             exceptions.handle(self.request,
                               _('Unable to retrieve package details.'),
                               redirect=reverse(INDEX_URL))
@@ -631,7 +610,7 @@ def download_packge(request, app_name, app_id):
         return response
     except exc.HTTPException:
         LOG.exception(_('Something went wrong during package downloading'))
-        redirect = reverse('horizon:murano:packages:index')
+        redirect = reverse('horizon:app-catalog:packages:index')
         exceptions.handle(request,
                           _('Unable to download package.'),
                           redirect=redirect)
