@@ -10,11 +10,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import contextlib
 import json
 import logging
 import os
+import six
 import six.moves.urllib.parse as urlparse
-import sys
 import testtools
 import time
 import uuid
@@ -23,13 +24,13 @@ from glanceclient import client as gclient
 from keystoneauth1.identity import v3
 from keystoneauth1 import session as ks_session
 from keystoneclient.v3 import client as ks_client
+from muranoclient.common import exceptions as muranoclient_exc
 from muranoclient.glance import client as glare_client
 import muranoclient.v1.client as mclient
 from oslo_log import handlers
 from oslo_log import log
 from selenium.common import exceptions as exc
 from selenium import webdriver
-from selenium.webdriver.common import action_chains
 import selenium.webdriver.common.by as by
 from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.support import ui
@@ -42,19 +43,8 @@ logger = log.getLogger(__name__).logger
 logger.level = logging.DEBUG
 logger.addHandler(handlers.ColorHandler())
 
-if sys.version_info >= (2, 7):
-    class BaseDeps(testtools.TestCase):
-        pass
-else:
-    # Define asserts for python26
-    import unittest2
 
-    class BaseDeps(testtools.TestCase,
-                   unittest2.TestCase):
-        pass
-
-
-class UITestCase(BaseDeps):
+class UITestCase(testtools.TestCase):
     @classmethod
     def setUpClass(cls):
         auth = v3.Password(user_domain_name='Default',
@@ -87,24 +77,27 @@ class UITestCase(BaseDeps):
     def setUp(self):
         super(UITestCase, self).setUp()
 
-        self.driver = webdriver.Firefox()
+        # Enables zip files to be automatically saved to disk, without opening
+        # a browser dialog.
+        fp = webdriver.FirefoxProfile()
+        fp.set_preference("browser.download.folderList", 2)
+        fp.set_preference("browser.download.manager.showWhenStarting", False)
+        fp.set_preference("browser.download.dir", os.getcwd())
+        fp.set_preference("browser.helperApps.neverAsk.saveToDisk",
+                          "application/octet-stream")
+
+        self.driver = webdriver.Firefox(firefox_profile=fp)
         self.addCleanup(self.driver.quit)
         self.driver.maximize_window()
-        self.driver.get(cfg.common.horizon_url + '/murano/environments')
+        self.driver.get(cfg.common.horizon_url + '/app-catalog/environments')
         self.driver.implicitly_wait(30)
         self.addOnException(self.take_screenshot)
         self.log_in()
-        self.projects_to_delete = []
         self.switch_to_project(cfg.common.tenant)
 
     def tearDown(self):
         super(UITestCase, self).tearDown()
 
-        self.switch_to_project(cfg.common.tenant)
-
-        for project_id in self.projects_to_delete:
-            self.keystone_client.projects.delete(project_id)
-
         for env in self.murano_client.environments.list():
             self.remove_environment(env.id)
 
@@ -164,13 +157,6 @@ class UITestCase(BaseDeps):
                      if tenant.name == name]
         return tenant_id[0]
 
-    def create_project(self, name):
-        project = self.keystone_client.projects.create(
-            name=name, domain="default", description="For Test Purposes",
-            enabled=True)
-        self.projects_to_delete.append(project.id)
-        return project.id
-
     def add_user_to_project(self, project_id, user_id, user_role=None):
         if not user_role:
             roles = self.keystone_client.roles.list()
@@ -218,7 +204,7 @@ class UITestCase(BaseDeps):
         self.fill_field(by.By.ID, 'id_username', username)
         self.fill_field(by.By.ID, 'id_password', password)
         self.driver.find_element_by_xpath("//button[@type='submit']").click()
-        murano = self.driver.find_element_by_xpath(consts.Applications)
+        murano = self.driver.find_element_by_xpath(consts.AppCatalog)
         if 'collapsed' in murano.get_attribute('class'):
             murano.click()
 
@@ -227,9 +213,11 @@ class UITestCase(BaseDeps):
             by.By.XPATH, "//ul[contains(@class, 'navbar-right')]")
         user_menu.find_element(
             by.By.XPATH, ".//span[@class='user-name']").click()
-        user_menu.find_element(by.By.LINK_TEXT, 'Sign Out').click()
+        user_menu.find_element(by.By.PARTIAL_LINK_TEXT, 'Sign Out').click()
 
     def fill_field(self, by_find, field, value):
+        self.check_element_on_page(by_find, field)
+        self.wait_element_is_clickable(by_find, field)
         self.driver.find_element(by=by_find, value=field).clear()
         self.driver.find_element(by=by_find, value=field).send_keys(value)
 
@@ -241,10 +229,14 @@ class UITestCase(BaseDeps):
         return path.split('__')[-1]
 
     def select_and_click_action_for_app(self, action, app):
-        self.driver.find_element_by_xpath(
-            "//*[@href='{0}/murano/catalog/{1}/{2}']".format(self.url_prefix,
-                                                             action,
-                                                             app)).click()
+        self.check_element_on_page(
+            by.By.XPATH,
+            "//*[@href='{0}/app-catalog/catalog/{1}/{2}']".format(
+                self.url_prefix, action, app))
+        self.wait_element_is_clickable(
+            by.By.XPATH,
+            "//*[@href='{0}/app-catalog/catalog/{1}/{2}']".format(
+                self.url_prefix, action, app)).click()
 
     def go_to_submenu(self, link):
         element = self.wait_element_is_clickable(by.By.PARTIAL_LINK_TEXT, link)
@@ -261,7 +253,6 @@ class UITestCase(BaseDeps):
             by.By.XPATH, getattr(consts, menu))
         if 'collapsed' in el.get_attribute('class'):
             el.click()
-        self.wait_for_sidebar_is_loaded()
 
     def select_from_list(self, list_name, value, sec=10):
         locator = (by.By.XPATH,
@@ -276,16 +267,16 @@ class UITestCase(BaseDeps):
             ui.WebDriverWait(self.driver, sec).until(
                 EC.presence_of_element_located((method, value)))
         except exc.TimeoutException:
-            self.fail("Element {0} is not preset on the page".format(value))
+            self.fail("Element {0} is not present on the page".format(value))
 
-    def check_element_not_on_page(self, method, value):
-        self.driver.implicitly_wait(3)
+    def check_element_not_on_page(self, method, value, sec=3):
+        self.driver.implicitly_wait(sec)
         present = True
         try:
             self.driver.find_element(method, value)
         except (exc.NoSuchElementException, exc.ElementNotVisibleException):
             present = False
-        self.assertFalse(present, u"Element {0} is preset on the page"
+        self.assertFalse(present, u"Element {0} is present on the page"
                                   " while it should't".format(value))
         self.driver.implicitly_wait(30)
 
@@ -295,7 +286,7 @@ class UITestCase(BaseDeps):
             ui.WebDriverWait(self.driver, sec).until(
                 EC.presence_of_element_located(locator))
         except exc.TimeoutException:
-            self.fail("Alert is not preset on the page")
+            self.fail("Alert is not present on the page")
 
         self.assertIn(message, self.driver.find_element(*locator).text)
 
@@ -310,38 +301,12 @@ class UITestCase(BaseDeps):
         self.driver.find_element_by_id(consts.ConfirmCreateEnvironment).click()
         self.wait_for_alert_message()
 
-    def delete_environment(self, env_name):
-        self.select_action_for_environment(env_name, 'delete')
+    def delete_environment(self, env_name, from_detail_view=False):
+        if not from_detail_view:
+            self.select_action_for_environment(env_name, 'delete')
         self.driver.find_element_by_xpath(consts.ConfirmDeletion).click()
         self.wait_for_alert_message()
 
-    def edit_environment(self, old_name, new_name):
-        el_td = self.driver.find_element_by_css_selector(
-            'tr[data-display="{0}"] '.format(old_name) +
-            'td[data-cell-name="name"]')
-        el_pencil = el_td.find_element_by_css_selector(
-            'button.ajax-inline-edit')
-
-        # hover to make pencil visible
-        hover = action_chains.ActionChains(self.driver).move_to_element(el_td)
-        hover.perform()
-        el_pencil.click()
-
-        # fill in inline input
-        el_inline_input = self.driver.find_element_by_css_selector(
-            'tr[data-display="{0}"] '.format(old_name) +
-            'td[data-cell-name="name"] .inline-edit-form input')
-        el_inline_input.clear()
-        el_inline_input.send_keys(new_name)
-
-        # click submit
-        el_submit = self.driver.find_element_by_css_selector(
-            'tr[data-display="{0}"] '.format(old_name) +
-            'td[data-cell-name="name"] .inline-edit-actions' +
-            ' button[type="submit"]')
-        el_submit.click()
-        # there is no alert message
-
     def select_action_for_environment(self, env_name, action):
         element_id = self.get_element_id(env_name)
         more_button = consts.More.format('environments', element_id)
@@ -355,6 +320,15 @@ class UITestCase(BaseDeps):
         ui.WebDriverWait(self.driver, sec).until(
             EC.presence_of_element_located(locator))
 
+    def wait_for_alert_message_to_disappear(self, sec=10):
+        # The alert message pops up directly over the delete environment
+        # button, causing click issues. So we must wait for the alert message
+        # to completely disappear before clicking the button.
+        locator = (by.By.CSS_SELECTOR, 'div.alert-success')
+        logger.debug("Waiting for a success message to disappear")
+        ui.WebDriverWait(self.driver, sec).until(
+            EC.invisibility_of_element_located(locator))
+
     def wait_for_error_message(self, sec=20):
         locator = (by.By.CSS_SELECTOR, 'div.alert-danger > p')
         logger.debug("Waiting for an error message")
@@ -369,40 +343,53 @@ class UITestCase(BaseDeps):
     def wait_for_sidebar_is_loaded(self, sec=10):
         ui.WebDriverWait(self.driver, sec).until(
             EC.presence_of_element_located(
-                (by.By.CSS_SELECTOR, "div#sidebar li.active")))
+                (by.By.CSS_SELECTOR, "nav#sidebar a.active")))
         time.sleep(0.5)
 
+    @contextlib.contextmanager
+    def wait_for_page_reload(self, sec=10):
+        old_page = self.driver.find_element_by_tag_name('html')
+        yield
+        ui.WebDriverWait(self, sec).until(EC.staleness_of(old_page))
+
 
 class PackageBase(UITestCase):
     @classmethod
     def setUpClass(cls):
         super(PackageBase, cls).setUpClass()
-        cls.mockapp_id = utils.upload_app_package(
-            cls.murano_client,
+        cls.packages = []
+        cls.mockapp_id = cls.upload_package(
             "MockApp",
             {"categories": ["Web"], "tags": ["tag"]})
-        cls.postgre_id = utils.upload_app_package(
-            cls.murano_client,
+        cls.postgre_id = cls.upload_package(
             "PostgreSQL",
             {"categories": ["Databases"], "tags": ["tag"]})
-        cls.hot_app_id = utils.upload_app_package(
-            cls.murano_client,
+        cls.hot_app_id = cls.upload_package(
             "HotExample",
             {"tags": ["hot"]}, hot=True)
-        cls.deployingapp_id = utils.upload_app_package(
-            cls.murano_client,
+        cls.deployingapp_id = cls.upload_package(
             "DeployingApp",
             {"categories": ["Web"], "tags": ["tag"]},
             hot=False,
             package_dir=consts.DeployingPackageDir)
 
+    @classmethod
+    def upload_package(cls, name, data, **kwargs):
+        package = utils.upload_app_package(cls.murano_client, name, data,
+                                           **kwargs)
+        cls.packages.append(package)
+        return package
+
     @classmethod
     def tearDownClass(cls):
         super(PackageBase, cls).tearDownClass()
-        cls.murano_client.packages.delete(cls.mockapp_id)
-        cls.murano_client.packages.delete(cls.postgre_id)
-        cls.murano_client.packages.delete(cls.hot_app_id)
-        cls.murano_client.packages.delete(cls.deployingapp_id)
+        # In case dynamically created packages are deleted at test level,
+        # ignore not found errors below.
+        for package in cls.packages:
+            try:
+                cls.murano_client.packages.delete(package)
+            except muranoclient_exc.HTTPNotFound:
+                pass
 
 
 class ImageTestCase(PackageBase):
@@ -410,48 +397,57 @@ class ImageTestCase(PackageBase):
     def setUpClass(cls):
         super(ImageTestCase, cls).setUpClass()
         glance_endpoint = cls.service_catalog.url_for(service_type='image')
-        cls.glance = gclient.Client('1', endpoint=glance_endpoint,
+        cls.glance = gclient.Client('2', endpoint=glance_endpoint,
                                     session=cls.keystone_client.session)
-        cls.image_title = 'New Image ' + str(time.time())
-        cls.image = cls.upload_image(cls.image_title)
 
-    @classmethod
-    def tearDownClass(cls):
-        super(ImageTestCase, cls).tearDownClass()
-        cls.glance.images.delete(cls.image.id)
+    def setUp(self):
+        super(ImageTestCase, self).setUp()
+        self.image_title = self.gen_random_resource_name('default-image', 15)
+        self.image = self.upload_image(self.image_title)
+
+    def tearDown(self):
+        super(ImageTestCase, self).tearDown()
+        self.glance.images.delete(self.image.id)
 
     @classmethod
     def upload_image(cls, title):
         try:
-            property = {'murano_image_info': json.dumps({'title': title,
-                                                         'type': 'linux'})}
-
+            murano_property = json.dumps({'title': title, 'type': 'linux'})
             image = cls.glance.images.create(name='TestImage',
                                              disk_format='qcow2',
-                                             size=0,
-                                             is_public=True,
-                                             properties=property)
-        except Exception as e:
+                                             container_format='bare',
+                                             is_public='True',
+                                             murano_image_info=murano_property)
+            image_data = six.BytesIO(None)
+            cls.glance.images.upload(image['id'], image_data)
+        except Exception:
             logger.error("Unable to create or update image in Glance")
-            raise e
+            raise
         return image
 
     def select_and_click_element(self, element):
         self.driver.find_element_by_xpath(
             ".//*[@value = '{0}']".format(element)).click()
 
-    def repair_image(self):
-        self.driver.find_element_by_id(
-            'marked_images__action_mark_image').click()
-        self.select_from_list('image', self.image.id)
-        self.fill_field(by.By.ID, 'id_title', self.image_title)
-        self.select_from_list('type', 'linux')
-        self.select_and_click_element('Mark Image')
-        self.check_element_on_page(by.By.XPATH,
-                                   consts.TestImage.format(self.image_title))
-
 
 class FieldsTestCase(PackageBase):
+    def check_js_error_message_is_present(self, error_message):
+        self.driver.implicitly_wait(2)
+        self.driver.find_element_by_xpath(
+            consts.JsErrorMessage.format(error_message))
+        self.driver.implicitly_wait(30)
+
+    def check_js_error_message_is_absent(self, error_message):
+        self.driver.implicitly_wait(2)
+        try:
+            self.driver.find_element_by_xpath(
+                consts.ErrorMessage.format(error_message))
+        except (exc.NoSuchElementException, exc.ElementNotVisibleException):
+            logger.info("Message {0} is not"
+                        " present on the page".format(error_message))
+
+        self.driver.implicitly_wait(30)
+
     def check_error_message_is_present(self, error_message):
         self.driver.find_element_by_xpath(consts.ButtonSubmit).click()
         self.driver.find_element_by_xpath(
@@ -472,12 +468,18 @@ class FieldsTestCase(PackageBase):
 
 
 class ApplicationTestCase(ImageTestCase):
-    def delete_component(self, component_name):
-        component_id = self.get_element_id(component_name)
-        self.driver.find_element_by_id(
-            'services__row_{0}__action_delete'.format(component_id)).click()
+    def delete_component(self, component_name=None):
+        if component_name:
+            component_id = self.get_element_id(component_name)
+            btn = self.wait_element_is_clickable(
+                by.By.ID, 'services__row_{0}__action_delete'.format(
+                    component_id))
+        else:
+            btn = self.wait_element_is_clickable(by.By.CSS_SELECTOR,
+                                                 consts.DeleteComponent)
+        btn.click()
         el = self.wait_element_is_clickable(by.By.LINK_TEXT,
-                                            'Delete Component')
+                                            'Delete Component', sec=30)
         el.click()
         self.wait_for_alert_message()
 
@@ -519,7 +521,8 @@ class ApplicationTestCase(ImageTestCase):
         self.wait_for_alert_message()
 
     def add_app_to_env(self, app_id, app_name='TestApp', env_id=None):
-        self.go_to_submenu('Browse')
+        self.navigate_to('Browse')
+        self.go_to_submenu('Browse Local')
         if env_id:
             action = 'add'
             app = '{0}/{1}'.format(app_id, env_id)
@@ -529,18 +532,59 @@ class ApplicationTestCase(ImageTestCase):
         self.select_and_click_action_for_app(action, app)
         field_id = "{0}_0-name".format(app_id)
         self.fill_field(by.By.ID, field_id, value=app_name)
-        self.driver.find_element_by_xpath(consts.ButtonSubmit).click()
-        self.driver.find_element_by_xpath(consts.InputSubmit).click()
+        self.wait_element_is_clickable(by.By.XPATH,
+                                       consts.ButtonSubmit).click()
+        self.wait_element_is_clickable(by.By.XPATH,
+                                       consts.InputSubmit).click()
         self.select_from_list('osImage', self.image.id)
 
-        self.driver.find_element_by_xpath(consts.InputSubmit).click()
         if env_id:
-            self.driver.find_element_by_xpath(consts.InputSubmit).click()
+            # If another app is added, then env_id is passed in. In this case,
+            # the 'Next' followed by 'Create' must be clicked.
+            self.check_element_on_page(by.By.CSS_SELECTOR,
+                                       consts.NextWizardSubmit)
+            self.wait_element_is_clickable(
+                by.By.CSS_SELECTOR, consts.NextWizardSubmit).click()
+            self.check_element_on_page(by.By.CSS_SELECTOR,
+                                       consts.CreateWizardSubmit)
+            self.wait_element_is_clickable(
+                by.By.CSS_SELECTOR, consts.CreateWizardSubmit).click()
+
             self.wait_element_is_clickable(by.By.ID, consts.AddComponent)
             self.check_element_on_page(by.By.LINK_TEXT, app_name)
         else:
+            # Otherwise, only 'Create' needs to be clicked.
+            self.check_element_on_page(by.By.CSS_SELECTOR,
+                                       consts.CreateWizardSubmit)
+            self.wait_element_is_clickable(
+                by.By.CSS_SELECTOR, consts.CreateWizardSubmit).click()
             self.wait_for_alert_message()
 
+    def execute_action_from_table_view(self, env_name, table_action):
+        """Executes an action like Deploy or Delete from the table view.
+
+        Does not handle clicking on the confirmation modal that may appear.
+
+        Scenario:
+            1. Checks for the table drop-down button and then clicks it.
+            2. Checks for the table drop-down menu to appear.
+            3. Checks for the ``table_action`` button and then clicks it.
+        """
+        self.check_element_on_page(
+            by.By.XPATH, consts.TableDropdownBtn.format(env_name))
+        dropdown_btn = self.driver.find_element(
+            by.By.XPATH, consts.TableDropdownBtn.format(env_name))
+        dropdown_btn.click()
+        self.check_element_on_page(by.By.XPATH,
+                                   consts.TableDropdownMenu.format(env_name))
+        self.check_element_on_page(
+            by.By.XPATH, consts.TableDropdownAction.format(env_name,
+                                                           table_action))
+        action_btn = self.driver.find_element(
+            by.By.XPATH, consts.TableDropdownAction.format(env_name,
+                                                           table_action))
+        action_btn.click()
+
 
 class PackageTestCase(ApplicationTestCase):
     @classmethod
