@@ -1,3 +1,15 @@
+#    Licensed under the Apache License, Version 2.0 (the "License"); you may
+#    not use this file except in compliance with the License. You may obtain
+#    a copy of the License at
+#
+#         http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#    License for the specific language governing permissions and limitations
+#    under the License.
+
 import json
 import logging
 import os
@@ -5,7 +17,9 @@ import yaml
 import zipfile
 
 from oslo_log import log
+from selenium.webdriver.support import wait
 
+import config.config as cfg
 from muranodashboard.tests.functional import consts
 
 log = log.getLogger(__name__).logger
@@ -31,15 +45,18 @@ class ImageException(Exception):
         return self._error_string
 
 
-def upload_app_package(client, app_name, data, hot=False):
+def upload_app_package(client, app_name, data, hot=False,
+                       package_dir=consts.PackageDir, **manifest_kwargs):
     try:
         if not hot:
-            manifest = os.path.join(consts.PackageDir, 'manifest.yaml')
-            archive = compose_package(app_name, manifest, consts.PackageDir)
+            manifest = os.path.join(package_dir, 'manifest.yaml')
+            archive = compose_package(app_name, manifest, package_dir,
+                                      **manifest_kwargs)
         else:
             manifest = os.path.join(consts.HotPackageDir, 'manifest.yaml')
             archive = compose_package(app_name, manifest,
-                                      consts.HotPackageDir, hot=True)
+                                      consts.HotPackageDir, hot=True,
+                                      **manifest_kwargs)
         package = client.packages.create(data, {app_name: open(archive, 'rb')})
         return package.id
     finally:
@@ -48,7 +65,7 @@ def upload_app_package(client, app_name, data, hot=False):
 
 
 def compose_package(app_name, manifest, package_dir,
-                    require=None, archive_dir=None, hot=False):
+                    require=None, archive_dir=None, hot=False, **kwargs):
     """Composes a murano package
 
     Composes package `app_name` with `manifest` file as a template for the
@@ -69,6 +86,8 @@ def compose_package(app_name, manifest, package_dir,
             mfest_copy['Classes'] = {fqn: 'mock_muranopl.yaml'}
         if require:
             mfest_copy['Require'] = require
+        if kwargs:
+            mfest_copy.update(kwargs)
         f.write(yaml.dump(mfest_copy, default_flow_style=False))
 
     name = app_name + '.zip'
@@ -95,3 +114,26 @@ def compose_bundle(bundle_path, app_names):
         bundle['Packages'].append({'Name': app_name})
     with open(bundle_path, 'w') as f:
         f.write(json.dumps(bundle))
+
+
+def glare_enabled():
+    return cfg.common.packages_service == "glare"
+
+
+class wait_for_page_change(object):
+    """Waits until current page change in a browser."""
+    def __init__(self, driver, timeout=30):
+        self.driver = driver
+        self.timeout = timeout
+
+    def __enter__(self):
+        self.old_page = self.driver.find_element_by_tag_name('html')
+
+    def page_has_loaded(self):
+        new_page = self.driver.find_element_by_tag_name('html')
+        return new_page.id != self.old_page.id
+
+    def __exit__(self, *_):
+        wait.WebDriverWait(self.driver, timeout=self.timeout).until(
+            lambda _: self.page_has_loaded()
+        )
