@@ -18,26 +18,22 @@ from muranodashboard.catalog import tabs
 from openstack_dashboard.test import helpers
 
 
-class TestLicenseTab(helpers.APITestCase):
-    def setUp(self):
-        """Set up the Form data."""
-        super(TestLicenseTab, self).setUp()
-
+class TestLicenseTab(helpers.APIMockTestCase):
     @mock.patch('muranodashboard.catalog.tabs.services')
     def test_license(self, mock_services):
         """Check that a license is returned."""
 
         # Fake the services.get_app_forms() call.
-        m = mock.MagicMock()
+        m = mock.Mock()
         m.base_fields = {
-            'license': mock.MagicMock(
+            'license': mock.Mock(
                 description='Lorem ipsum dolor sit '
                             'amet, consectetur adipiscing elit.')
         }
         mock_services.get_app_forms.return_value = [('', m)]
 
         # Fake an application object, needed when instantiating tabs.
-        app = mock.MagicMock()
+        app = mock.Mock()
         app.id = 1
 
         group = tabs.ApplicationTabs(self.request, application=app)
@@ -54,12 +50,12 @@ class TestLicenseTab(helpers.APITestCase):
         """Check that no license is returned."""
 
         # Fake the services.get_app_forms() call.
-        m = mock.MagicMock()
+        m = mock.Mock()
         m.base_fields = {}
         mock_services.get_app_forms.return_value = [('', m)]
 
         # Fake an application object, needed when instantiating tabs.
-        app = mock.MagicMock()
+        app = mock.Mock()
         app.id = 1
 
         group = tabs.ApplicationTabs(self.request, application=app)
@@ -70,18 +66,14 @@ class TestLicenseTab(helpers.APITestCase):
         self.assertEqual('', l.app.license)
 
 
-class TestRequirementsTab(helpers.APITestCase):
-    def setUp(self):
-        """Set up the Form data."""
-        super(TestRequirementsTab, self).setUp()
-
+class TestRequirementsTab(helpers.APIMockTestCase):
     @mock.patch('muranodashboard.catalog.tabs.services')
     def test_requirements(self, mock_services):
         """Check that requirements are returned."""
 
-        m = mock.MagicMock()
+        m = mock.Mock()
         m.base_fields = {
-            'flavor': mock.MagicMock(requirements={
+            'flavor': mock.Mock(requirements={
                 'min_disk': 10,
                 'min_vcpus': 2,
                 'min_memory_mb': 2048,
@@ -92,7 +84,7 @@ class TestRequirementsTab(helpers.APITestCase):
         }
         mock_services.get_app_forms.return_value = [('', m)]
 
-        app = mock.MagicMock()
+        app = mock.Mock()
         app.id = 1
 
         group = tabs.ApplicationTabs(self.request, application=app)
@@ -121,11 +113,11 @@ class TestRequirementsTab(helpers.APITestCase):
     def test_no_requirements(self, mock_services):
         """Check that no requirements are returned."""
 
-        m = mock.MagicMock()
+        m = mock.Mock()
         m.base_fields = {}
         mock_services.get_app_forms.return_value = [('', m)]
 
-        app = mock.MagicMock()
+        app = mock.Mock()
         app.id = 1
 
         group = tabs.ApplicationTabs(self.request, application=app)
